// ///////////////////////////////////////////////////////////////////////////
//
// GNRE - Classe de Funcoes Compativeis para NFe contemplando GNRE
//
// Autor: Jose Carlos da Rocha
// Data: 26/12/2015
// Email: irochinha@hotmail.com.br
// MSN: fivolution@hotmail.com
// Linguagem: xBase
// Prefixo: GNRE
// Plataformas: DOS, Windows
// Requerimentos: Harbour/xHarbour
//
// Fonte: http://www.nfp.fazenda.sp.gov.br/leg03_port102_07.shtm
//
// ///////////////////////////////////////////////////////////////////////////

#include "hbclass.ch"
#include "common.ch"

FUNCTION Main()

 oGNRE := tGNRE():new()

* oGNRE:nLote       := 0 // Se numero de lote for igual a ZERO sera gerada uma guia unica, o contratio gera lote com varias guias
 oGNRE:nLote       := 1 // Se numero de lote for igual a ZERO sera gerada uma guia unica, o contratio gera lote com varias guias
 oGNRE:Homologacao     := .T. // Se homologacao direciona para webservices especificos

 oGNRE:c01_UfFavorecida    := 26 // Contm a sigla da UF favorecida. Campo com 2 dgitos. (Ver ANEXO 1, na aba 'Anexos')
 oGNRE:c02_receita     := 1000099 // Contm o cdigo da receita. Campo numrico com 6 dgitos. (Consultar na aba 'Regras de Preenchimento')
 oGNRE:c25_detalhamentoReceita  := 10101010 // Contm o cdigo do detalhamento da receita. Campo numrico. A quantidade de dgitos corresponde a quantidade de dgitos do cdigo. (Consultar na aba 'Regras de Preenchimento').
 oGNRE:c26_produto     := "TESTE DE PROD" // Contm o cdigo do produto. Campo numrico. A quantidade de dgitos corresponde a quantidade de dgitos do cdigo. (Consultar na aba 'Regras de Preenchimento').
 oGNRE:c27_tipoIdentificacaoEmitente := 1 // Contm o cdigo do tipo de identificao do contribuinte emitente (Responsvel pelo Pagamento do Tributo). Campo numrico com 1 dgito. (Ver ANEXO 2, na aba 'Anexos'). Caso a identificao seja pela inscrio estadual, esse campo e sua tag podero ser omitidos.
 oGNRE:c03_idContribuinteEmitente  := 41819055000105 // Contm o nmero do documento de identificao do contribuinte emitente. O CPF/ CNPJ no dever conter espaos, pontos ou traos e apenas um documento poder ser colocado no arquivo. Caso a identificao seja pela inscrio estadual, esse campo e sua tag podero ser omitidos. Caso a identificao seja pelo CNPJ, a tag de CPF e de Inscrio Estadual podero ser omitidas. Caso a identificao seja pelo CPF, os campos e as tags de CNPJ e de Inscrio Estadual podero ser omitidos.
 // Exemplos:
 // CNPJ : <CNPJ>11111111111111</CNPJ>
 // CPF : <CPF>11111111111</CPF>
 oGNRE:c28_tipoDocOrigem    := 10 // Contm o cdigo do tipo de documento de origem. Campo numrico. A quantidade de dgitos corresponde a quantidade de dgitos do cdigo. (Consultar na aba 'Regras de Preenchimento'). Caso a receita no exija documento de origem, esse campo e sua tag podero ser omitidos.
 oGNRE:c04_docOrigem     := 5656 // Contm o nmero contido no documento de origem. Campo numrico sem espaos, traos, pontos ou vrgulas. A quantidade de dgitos corresponde a quantidade de dgitos do nmero. Caso a receita no exija documento de origem, esse campo e sua tag podero ser omitidos.
 oGNRE:c06_valorPrincipal    := 10.99 // Contm o valor original da guia. Digitar apenas nmeros. Usar ponto (".") como separador de decimal, e este deve ter 2 nmeros. Caso a receita no exija valor principal, esse campo e sua tag podero ser omitidos.
 oGNRE:c10_valorTotal     := 12.52 // Contm valor total da guia (valor original + encargos). Digitar apenas nmeros. Usar ponto (".") como separador de decimal, e este deve ter 2 nmeros. Caso a receita no exija valor total, esse campo e sua tag podero ser omitidos. Para calcular os encargos, procurar informaes na Sefaz do Estado da UF favorecida.
 oGNRE:c14_dataVencimento    := "2015-05-01" // Contm a data que o contribuinte deve pagar o tributo de acordo com a legislao de cada UF. Formato: AAAA-MM-DD.
 oGNRE:c15_convenio     := 546456 // Contm o nmero do convnio. Sem espaos, traos, pontos ou vrgulas. Este campo no  obrigatrio e o tamanho do campo corresponde ao nmero do convnio. No caso de no ter essa informao, esse campo e sua tag podero ser omitidos.
 oGNRE:c16_razaoSocialEmitente  := "GNRE PHP EMITENTE" // Contm o nome da razo social do contribuinte emitente. No caso da identificao do contribuinte ser por inscrio estadual, esse campo e sua tag podero ser omitidos.
 oGNRE:c17_inscricaoEstadualEmitente := 56756 // Contm a inscrio estadual do contribuinte emitente na UF favorecida. Campo numrico sem espaos ou traos. Caso seja inscrito na UF Favorecida, preencher este campo. Caso contrrio, esse campo e sua tag podero ser omitidos.
 oGNRE:c18_enderecoEmitente   := "Queens St" // Contm o endereo do contribuinte emitente. No caso da identificao do contribuinte ser por inscrio estadual, esse campo e sua tag podero ser omitidos.
 oGNRE:c19_municipioEmitente   := 5300108 // Contm o cdigo do municpio de localizao do contribuinte emitente. Campo numrico. (Consultar no site do IBGE). O IBGE informar o cdigo do municpio no formato "EEmmmmd". Tirar os 2 primeiros nmeros que indicam o nmero do Estado e s colocar os 5 nmeros restantes, ficando no formato "mmmmd".
 // A cidade Recife tem o cdigo 2611606, voc tirar os dgitos "26" e colocar no arquivo de lote apenas os dgitos "11606".
 oGNRE:c20_ufEnderecoEmitente   := "DF" // Contm a UF de localizao do contribuinte emitente. Campo com 2 dgitos. (Ver ANEXO 1, na aba 'Anexos'). No caso da identificao do contribuinte ser por inscrio estadual, esse campo e sua tag podero ser omitidos.
 oGNRE:c21_cepEmitente    := "08215917" // Contm o CEP do contribuinte emitente com 8 dgitos. Digitar apenas nmeros. Esse campo no  obrigatrio. No caso da identificao do contribuinte ser por inscrio estadual, esse campo e sua tag podero ser omitidos.
 oGNRE:c22_telefoneEmitente   := 1199999999 // Contm o telefone do contribuinte emitente. Colocar o DDD e o nmero do telefone. Digitar apenas nmeros. Esse campo no  obrigatrio. No caso de no ter essa informao, esse campo e sua tag podero ser omitidos. Onde: 11 => DDD e 22222222 => Telefone
 oGNRE:c34_tipoIdentificacaoDestinatario := 1 // Contm o cdigo do tipo de identificao do contribuinte destinatrio. Campo numrico com 1 dgito. (Ver ANEXO 2, na aba 'Anexos'). Caso a receita no exija destinatrio, esse campo e sua tag podero ser omitidos.
 oGNRE:c35_idContribuinteDestinatario := 86268158000162 // Contm o nmero do documento de identificao do contribuinte destinatrio. O CPF/ CNPJ no dever conter espaos, pontos ou traos e apenas um documento poder ser colocado no arquivo. Caso a receita no exija destinatrio, esse campo e sua tag podero ser omitidos.
 oGNRE:c36_inscricaoEstadualDestinatario := 10809181 // Contm a inscrio estadual do contribuinte destinatrio. Campo numrico sem espaos ou traos. Caso seja inscrito na UF Favorecida, preencher este campo. Caso a receita no exija destinatrio, esse campo e sua tag podero ser omitidos.
 oGNRE:c37_razaoSocialDestinatario := "RAZAO SOCIAL GNRE PHP DESTINATARIO" // Contm o nome da firma ou a razo social do contribuinte destinatrio. Caso a receita no exija destinatrio, esse campo e sua tag podero ser omitidos.
 oGNRE:c38_municipioDestinatario  := 2702306 // Contm o municpio de localizao do contribuinte destinatrio. Campo numrico. (Consultar no site do IBGE). O IBGE informar o cdigo do municpio no formato "EEmmmmd". Tirar os 2 primeiros nmeros que indicam o nmero do Estado e s colocar os 5 nmeros restantes, ficando no formato "mmmmd".
 oGNRE:c33_dataPagamento    := "2015-11-30" // Contm a data que o contribuinte ir pagar o tributo.Formato: AAAA-MM-DD.
 oGNRE:c05_referencia     := "" // Contm as informaes do perodo de referncia. Caso a receita no exija perodo de referncia, esse campo e sua tag podero ser omitidos.
 oGNRE:mes       := "05" // Contm o ms de referncia da apurao. Campo numrico com 2 dgitos. (Ver ANEXO 3, na aba 'Anexos'). Caso a receita no exija ms de perodo de referncia, esse campo e sua tag podero ser omitidos.
 oGNRE:ano       := 2015 // Contm o ano de referncia da apurao. Campo numrico com 4 dgitos. Caso a receita no exija ano de perodo de referncia, esse campo e sua tag podero ser omitidos.
 oGNRE:parcela      := 2 // Contm o nmero da parcela do dbito. Campo numrico com tamanho mximo de 3 dgitos. Caso a receita no exija parcela no perodo de referncia, esse campo e sua tag podero ser omitidos.
 oGNRE:periodo      := 2014 // Contm o cdigo do perodo. Campo numrico. A quantidade de dgitos corresponde a quantidade de dgitos do cdigo. (Consultar na aba 'Regras de Preenchimento'). Caso a receita no exija cdigo do perodo, esse campo e sua tag podero ser omitidos
 // Nao importantes
 // c39_camposExtras      := "" // Lista de campos extras. Poder ter no mximo 3 campos extras por guia. Caso a receita no exija campos extras, esse campo e sua tag podero ser omitidos.
 // oGNRE:codigo       := 0 // Contm o cdigo do campo extra. Campo numrico. A quantidade de dgitos corresponde a quantidade de dgitos do cdigo. (Consultar na aba 'Regras de Preenchimento'). Caso a receita no exija campos extras, esse campo e sua tag podero ser omitidos.
 // oGNRE:tipo       := "" // Contm o tipo do campo extra. Campo com um caracter. (Consultar na aba 'Regras de Preenchimento'). Caso a receita no exija campos extras, esse campo e sua tag podero ser omitidos.
 // oGNRE:valor       := 0.00 // Contm o valor solicitado no campo extra. Se for do tipo Data 'D', o formato : AAAA-MM-DD. Se for do tipo Nmerico 'N', usar ponto (".") como separador de decimal, e este deve ter 2 nmeros.
 // oGNRE:c42_identificadorGuia   := 0 // Contm o identificador da guia no lote. O valor deste campo dever ser informado pelo contribuinte para a identificao da guia. Esta informao  opcional e deve conter apenas nmeros (at 10 posies).

 // $lote = new Lote();
 // $lote->addGuia($guia);
 oGNRE:AdicionaGuia()

 ? oGNRE:GeraXml()

* oGNRE:Enviar()

 RETURN .T.

CREATE CLASS tGNRE

 VAR nLote
 VAR cFileName
 VAR Homologacao
 VAR oGNRERegistros
 VAR oGNRERegExtras
 VAR c01_UfFavorecida
 VAR c02_receita
 VAR c25_detalhamentoReceita
 VAR c26_produto
 VAR c27_tipoIdentificacaoEmitente
 VAR c03_idContribuinteEmitente
 VAR c28_tipoDocOrigem
 VAR c04_docOrigem
 VAR c06_valorPrincipal
 VAR c10_valorTotal
 VAR c14_dataVencimento
 VAR c15_convenio
 VAR c16_razaoSocialEmitente
 VAR c17_inscricaoEstadualEmitente
 VAR c18_enderecoEmitente
 VAR c19_municipioEmitente
 VAR c20_ufEnderecoEmitente
 VAR c21_cepEmitente
 VAR c22_telefoneEmitente
 VAR c34_tipoIdentificacaoDestinatario
 VAR c35_idContribuinteDestinatario
 VAR c36_inscricaoEstadualDestinatario
 VAR c37_razaoSocialDestinatario
 VAR c38_municipioDestinatario
 VAR c33_dataPagamento
 VAR c05_referencia
 VAR mes
 VAR ano
 VAR parcela
 VAR periodo
 //
 METHOD New()
 METHOD AdicionaGuia()
 METHOD GeraXml()
 //
 METHOD Enviar()
 METHOD Consulta()

END CLASS

METHOD New( cGNREFileName ) CLASS tGNRE
   if cGNREFileName == nil
      ::cFileName := "GNRE.TXT"
   else
      ::cFileName := cGNREFileName
   endif

* ::nLote := 0
 ::nLote := 1
 ::Homologacao := .F.
 ::oGNRERegistros := {}
 ::oGNRERegExtras := {}

 RETURN SELF

METHOD AdicionaGuia() CLASS tGNRE
 AAdd( ::oGNRERegistros, SELF )
 RETURN NIL

METHOD GeraXml() CLASS tGNRE

 LOCAL oGNRE
 local nGNRE := fCreate( ::cFileName )

 cResultado := "" // [<?xml version="1.0" encoding="utf-8"?>] + CRLF
 
 IF ::nLote > 0
    cResultado += [ <TLote_GNRE xmlns="http://www.gnre.pe.gov.br">] + CRLF
    cResultado += [  <guias>] + CRLF
 ENDIF

 FOR EACH oGNRE IN ::oGNRERegistros
   // cResultado += [   <guias>] + CRLF
   cResultado += [   <TDadosGNRE>] + CRLF
   cResultado += [    <c01_UfFavorecida>] + AllTrim( Str( oGNRE:c01_UFFavorecida ) ) + [</c01_UfFavorecida>] + CRLF
   cResultado += [    <c02_receita>] + AllTrim( Str( oGNRE:c02_Receita ) ) + [</c02_receita>] + CRLF
   cResultado += [    <c25_detalhamentoReceita>] + AllTrim( Str( oGNRE:c25_detalhamentoReceita ) ) + [</c25_detalhamentoReceita>] + CRLF
   cResultado += [    <c26_produto>] + AllTrim( oGNRE:c26_Produto ) + [</c26_produto>] + CRLF
   cResultado += [    <c27_tipoIdentificacaoEmitente>] + AllTrim( Str( oGNRE:c27_TipoIdentificacaoEmitente ) ) + [</c27_tipoIdentificacaoEmitente>] + CRLF
   cResultado += [    <c03_idContribuinteEmitente>] + CRLF
   cResultado += [     <CNPJ>] + AllTrim( Str( oGNRE:c03_idContribuinteEmitente ) ) + [</CNPJ>] + CRLF
   cResultado += [    </c03_idContribuinteEmitente>] + CRLF
   cResultado += [    <c28_tipoDocOrigem>] + AllTrim( Str( oGNRE:c28_TipoDocOrigem ) ) + [</c28_tipoDocOrigem>] + CRLF
   cResultado += [    <c04_docOrigem>] + AllTrim( Str( oGNRE:c04_DocOrigem ) ) + [</c04_docOrigem>] + CRLF
   cResultado += [    <c06_valorPrincipal>] + AllTrim( Str( oGNRE:c06_ValorPrincipal, 2 ) ) + [</c06_valorPrincipal>] + CRLF
   cResultado += [    <c10_valorTotal>] + AllTrim( Str( oGNRE:c10_ValorTotal, 2 ) ) + [</c10_valorTotal>] + CRLF
   cResultado += [    <c14_dataVencimento>] + AllTrim( oGNRE:c14_DataVencimento ) + [</c14_dataVencimento>] + CRLF
   cResultado += [    <c15_convenio>] + AllTrim( Str( oGNRE:c15_Convenio ) ) + [</c15_convenio>] + CRLF
   cResultado += [    <c16_razaoSocialEmitente>] + AllTrim( oGNRE:c16_RazaoSocialEmitente ) + [</c16_razaoSocialEmitente>] + CRLF
   cResultado += [    <c17_inscricaoEstadualEmitente>] + AllTrim( Str( oGNRE:c17_inscricaoEstadualEmitente ) ) + [</c17_inscricaoEstadualEmitente>] + CRLF
   cResultado += [    <c18_enderecoEmitente>] + AllTrim( oGNRE:c18_EnderecoEmitente ) + [</c18_enderecoEmitente>] + CRLF
   cResultado += [    <c19_municipioEmitente>] + AllTrim( Str( oGNRE:c19_MunicipioEmitente ) ) + [</c19_municipioEmitente>] + CRLF
   cResultado += [    <c20_ufEnderecoEmitente>] + AllTrim( oGNRE:c20_UFEnderecoEmitente ) + [</c20_ufEnderecoEmitente>] + CRLF
   cResultado += [    <c21_cepEmitente>] + AllTrim( oGNRE:c21_CepEmitente ) + [</c21_cepEmitente>] + CRLF
   cResultado += [    <c22_telefoneEmitente>] + AllTrim( Str( oGNRE:c22_TelefoneEmitente ) ) + [</c22_telefoneEmitente>] + CRLF
   cResultado += [    <c34_tipoIdentificacaoDestinatario>] + AllTrim( Str( oGNRE:c34_TipoIdentificacaoDestinatario ) ) + [</c34_tipoIdentificacaoDestinatario>] + CRLF
   cResultado += [    <c35_idContribuinteDestinatario>] + CRLF
   cResultado += [     <CNPJ>] + AllTrim( Str( oGNRE:c35_idContribuinteDestinatario ) ) + [</CNPJ>] + CRLF
   cResultado += [    </c35_idContribuinteDestinatario>] + CRLF
   cResultado += [    <c36_inscricaoEstadualDestinatario>] + AllTrim( Str( oGNRE:c36_InscricaoEstadualDestinatario ) ) + [</c36_inscricaoEstadualDestinatario>] + CRLF
   cResultado += [    <c37_razaoSocialDestinatario>] + AllTrim( oGNRE:c37_RazaoSocialDestinatario ) + [</c37_razaoSocialDestinatario>] + CRLF
   cResultado += [    <c38_municipioDestinatario>] + AllTrim( Str( oGNRE:c38_MunicipioDestinatario ) ) + [</c38_municipioDestinatario>] + CRLF
   cResultado += [    <c33_dataPagamento>] + AllTrim( oGNRE:c33_DataPagamento ) + [</c33_dataPagamento>] + CRLF
   cResultado += [    <c05_referencia>] + CRLF
   cResultado += [     <mes>] + AllTrim( oGNRE:c05_Referencia ) + [</mes>] + CRLF
   cResultado += [     <ano>] + AllTrim( Str( ::Ano ) ) + [</ano>] + CRLF
   cResultado += [     <parcela>] + AllTrim( Str( ::Parcela ) ) + [</parcela>] + CRLF
   cResultado += [     <periodo>] + AllTrim( Str( ::Periodo ) ) + [</periodo>] + CRLF
   cResultado += [    </c05_referencia>] + CRLF
 /*
 cResultado += [   <c39_camposExtras>]
 cResultado += [   <campoExtra>]
 cResultado += [    <codigo>16</codigo>]
 cResultado += [    <tipo>T</tipo>]
 cResultado += [    <valor>1200012</valor>]
 cResultado += [   </campoExtra>]
 cResultado += [   <campoExtra>]
 cResultado += [    <codigo>15</codigo>]
 cResultado += [    <tipo>D</tipo>]
 cResultado += [    <valor>2015-03-02</valor>]
 cResultado += [   </campoExtra>]
 cResultado += [   <campoExtra>]
 cResultado += [    <codigo>10</codigo>]
 cResultado += [    <tipo>T</tipo>]
 cResultado += [    <valor>17.21</valor>]
 cResultado += [   </campoExtra>]
 cResultado += [   </c39_camposExtras>]
 */
   cResultado += [   </TDadosGNRE>] + CRLF
 NEXT

 if ::nLote > 0
   //
   cResultado += [  </guias>] + CRLF
   cResultado += [ </TLote_GNRE>] + CRLF
 ENDIF

  fWrite( nGNRE, cResultado, LEN(cResultado) )
  fClose( nGNRE )

 RETURN cResultado

METHOD Enviar() CLASS tGNRE

 LOCAL cResultado

 cURL := iif( ::Homologacao, "http://www.gnre-he.sp.gov.br", "http://www.gnre.sp.gov.br" )
 //
 cResultado := [<?xml version="1.0" encoding="utf-8"?>]
 //
 cResultado += [<soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">] + CRLF
 cResultado += [ <soap12:Header>] + CRLF
 cResultado += [ <gnreCabecMsg xmlns="http://www.gnre.pe.gov.br/wsdl/processar">] + CRLF
 cResultado += [  <versaoDados>1.00</versaoDados>] + CRLF
 cResultado += [ </gnreCabecMsg>] + CRLF
 cResultado += [ </soap12:Header>] + CRLF
 cResultado += [ <soap12:Body>] + CRLF
 cResultado += [ <gnreDadosMsg xmlns="http://www.gnre.pe.gov.br/webservice/GnreLoteRecepcao">] + CRLF
 cResultado += [  <TLote_GNRE xmlns="] + cURL + [">] + CRLF
 cResultado += [   <guias>] + CRLF
 cResultado += ::GeraXml() + CRLF
 cResultado += [   </guias>] + CRLF
 cResultado += [  </TLote_GNRE>] + CRLF
 cResultado += [ </gnreDadosMsg>] + CRLF
 cResultado += [ </soap12:Body>] + CRLF
 cResultado += [</soap12:Envelope>] + CRLF
 //
 // ?"Enviar", cResultado
 cResultado := GNREWebService( cResultado, "POST", cURL, "" ) // cSoapAction )

 RETURN cResultado

METHOD Consulta( cProtocolo, cUsuario, cSenha, cCNPJ, cCategoria ) CLASS tGNRE

 LOCAL cResultado

 //
 cResultado :=   [<?xml version="1.0" encoding="utf-8"?>]
 cResultado := ""
 //

 RETURN cResultado

FUNCTION GNREWebService( cXML, cMethod, cURL, cSoapAction )

 cMethod  := iif( cMethod == NIL, "POST", cMethod )
 cUrl    := "http://www.gnre.pe.gov.br/wsdl/processar"
 cSoapAction := cUrl
 oEDXSend  := win_OleCreateObject( "Microsoft.XMLHTTP" )
 oEDXSend:Open( cMethod, cURL, .F. )
 // oEDXSend:setrequestheader( "SOAPAction", cSoapAction )
 // oEDXSend:setrequestheader( "content-type", "text/xml" )
 oEDXSend:Send( cXml )
 cResultado := oEDXSend:ResponseText()

 RETURN cResultado

